@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2015 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -34,11 +34,11 @@
 #ifndef OPENVDB_MATH_MAT_HAS_BEEN_INCLUDED
 #define OPENVDB_MATH_MAT_HAS_BEEN_INCLUDED
 
-#include <math.h>
-#include <iostream>
-#include <boost/format.hpp>
-#include <openvdb/Exceptions.h>
 #include "Math.h"
+#include <openvdb/Exceptions.h>
+#include <boost/format.hpp>
+#include <cmath>
+#include <iostream>
 
 
 namespace openvdb {
@@ -72,6 +72,15 @@ public:
         }
     }
 
+    Mat& operator=(Mat const& src) {
+        if (&src != this) {
+            for (unsigned i = 0; i < numElements(); ++i) {
+                mm[i] = src.mm[i];
+            }
+        }
+        return *this;
+    }
+
     /// @return string representation of matrix
     /// Since output is multiline, optional indentation argument prefixes
     /// each newline with that much white space. It does not indent
@@ -136,6 +145,46 @@ public:
         is.read(reinterpret_cast<char*>(&mm), sizeof(T)*SIZE*SIZE);
     }
 
+    /// Return the maximum of the absolute of all elements in this matrix
+    T absMax() const {
+        T x = static_cast<T>(std::fabs(mm[0]));
+        for (unsigned i = 1; i < numElements(); ++i) {
+            x = std::max(x, static_cast<T>(std::fabs(mm[i])));
+        }
+        return x;
+    }
+
+    /// True if a Nan is present in this matrix
+    bool isNan() const { 
+        for (unsigned i = 0; i < numElements(); ++i) {
+            if (std::isnan(mm[i])) return true;
+        }
+        return false;
+    }
+
+    /// True if an Inf is present in this matrix
+    bool isInfinite() const {
+        for (unsigned i = 0; i < numElements(); ++i) {
+            if (std::isinf(mm[i])) return true;
+        }
+        return false;
+    }
+
+    /// True if no Nan or Inf values are present
+    bool isFinite() const {
+        for (unsigned i = 0; i < numElements(); ++i) {
+            if (!std::isfinite(mm[i])) return false;
+        }
+        return true;
+    }
+
+    /// True if all elements are exactly zero
+    bool isZero() const {
+        for (unsigned i = 0; i < numElements(); ++i) {
+            if (!isZero(mm[i])) return false;
+        }
+        return true;
+    }
 
 protected:
     T mm[SIZE*SIZE];
@@ -720,6 +769,40 @@ aim(const Vec3<typename MatType::value_type>& direction,
     return r;
 }
 
+/// @brief    This function snaps a specific axis to a specific direction,
+///           preserving scaling.
+/// @details  It does this using minimum energy, thus posing a unique solution if
+///           basis & direction aren't parallel.
+/// @note     @a direction need not be unit.
+template<class MatType>
+inline MatType
+snapMatBasis(const MatType& source, Axis axis, const Vec3<typename MatType::value_type>& direction)
+{
+    typedef typename MatType::value_type T;
+
+    Vec3<T> unitDir(direction.unit());
+    Vec3<T> ourUnitAxis(source.row(axis).unit());
+
+    // Are the two parallel?
+    T parallel = unitDir.dot(ourUnitAxis);
+
+    // Already snapped!
+    if (isApproxEqual(parallel, T(1.0))) return source;
+
+    if (isApproxEqual(parallel, T(-1.0))) {
+        OPENVDB_THROW(ValueError, "Cannot snap to inverse axis");
+    }
+
+    // Find angle between our basis and the one specified
+    T angleBetween(angle(unitDir, ourUnitAxis));
+    // Caclulate axis to rotate along
+    Vec3<T> rotationAxis = unitDir.cross(ourUnitAxis);
+
+    MatType rotation;
+    rotation.setToRotation(rotationAxis, angleBetween);
+
+    return source * rotation;
+}
 
 /// @brief Write 0s along Mat4's last row and column, and a 1 on its diagonal.
 /// @details Useful initialization when we're initializing just the 3x3 block.
@@ -737,94 +820,65 @@ padMat4(MatType& dest)
 
 /// @brief Solve for A=B*B, given A.
 /// @details Denman-Beavers square root iteration
-template <typename MatType>
+template<typename MatType>
 inline void
-sqrtSolve(const MatType &aA, MatType &aB, double aTol=0.01)
+sqrtSolve(const MatType& aA, MatType& aB, double aTol=0.01)
 {
-    unsigned int iterations = (unsigned int)(log(aTol)/log(0.5));
-    MatType Y[2];
-    MatType Z[2];
-    MatType invY;
-    MatType invZ;
+    unsigned int iterations = static_cast<unsigned int>(log(aTol)/log(0.5));
 
-    unsigned int current = 0;
-
-    Y[0]=aA;
+    MatType Y[2], Z[2];
+    Y[0] = aA;
     Z[0] = MatType::identity();
 
-    unsigned int iteration;
-    for (iteration=0; iteration<iterations; iteration++)
-    {
+    unsigned int current = 0;
+    for (unsigned int iteration=0; iteration < iterations; iteration++) {
         unsigned int last = current;
         current = !current;
 
-        invY = Y[last].inverse();
-        invZ = Z[last].inverse();
+        MatType invY = Y[last].inverse();
+        MatType invZ = Z[last].inverse();
 
-        Y[current]=0.5*(Y[last]+invZ);
-        Z[current]=0.5*(Z[last]+invY);
+        Y[current] = 0.5 * (Y[last] + invZ);
+        Z[current] = 0.5 * (Z[last] + invY);
     }
-
-    MatType &R = Y[current];
-
-    aB=R;
+    aB = Y[current];
 }
 
 
-template <typename MatType>
+template<typename MatType>
 inline void
-powSolve(const MatType &aA, MatType &aB, double aPower, double aTol=0.01)
+powSolve(const MatType& aA, MatType& aB, double aPower, double aTol=0.01)
 {
-    unsigned int iterations = (unsigned int)(log(aTol)/log(0.5));
+    unsigned int iterations = static_cast<unsigned int>(log(aTol)/log(0.5));
 
-    const bool inverted = ( aPower < 0.0 );
+    const bool inverted = (aPower < 0.0);
+    if (inverted) { aPower = -aPower; }
 
-    if (inverted) {
-        aPower = -aPower;
-    }
-
-    unsigned int whole = (unsigned int)aPower;
+    unsigned int whole = static_cast<unsigned int>(aPower);
     double fraction = aPower - whole;
 
-    MatType R;
-    R = MatType::identity();
-
+    MatType R = MatType::identity();
     MatType partial = aA;
 
     double contribution = 1.0;
-
-    unsigned int iteration;
-
-    for (iteration=0; iteration< iterations; iteration++)
-    {
+    for (unsigned int iteration = 0; iteration < iterations; iteration++) {
         sqrtSolve(partial, partial, aTol);
         contribution *= 0.5;
-
-        if (fraction>=contribution)
-        {
+        if (fraction >= contribution) {
             R *= partial;
-            fraction-=contribution;
+            fraction -= contribution;
         }
     }
 
     partial = aA;
-    while (whole)
-    {
-        if (whole & 1) {
-            R *= partial;
-        }
-        whole>>=1;
-        if(whole) {
-            partial*=partial;
-        }
+    while (whole) {
+        if (whole & 1) { R *= partial; }
+        whole >>= 1;
+        if (whole) { partial *= partial; }
     }
 
-    if (inverted) {
-        aB = R.inverse();
-    }
-    else {
-        aB = R;
-    }
+    if (inverted) { aB = R.inverse(); }
+    else { aB = R; }
 }
 
 
@@ -842,8 +896,8 @@ template<typename MatType>
 inline bool
 isInvertible(const MatType& m)
 {
-    typedef typename MatType::ValueType  value_type;
-    return !isApproxEqual(m.det(), (value_type)0);
+    typedef typename MatType::ValueType ValueType;
+    return !isApproxEqual(m.det(), ValueType(0));
 }
 
 
@@ -987,6 +1041,6 @@ polarDecomposition(const MatType& input, MatType& unitary,
 
 #endif // OPENVDB_MATH_MAT_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2015 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
