@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -31,10 +31,12 @@
 #ifndef OPENVDB_MATH_VEC3_HAS_BEEN_INCLUDED
 #define OPENVDB_MATH_VEC3_HAS_BEEN_INCLUDED
 
-#include <cmath>
 #include <openvdb/Exceptions.h>
 #include "Math.h"
 #include "Tuple.h"
+#include <cmath>
+#include <type_traits>
+
 
 namespace openvdb {
 OPENVDB_USE_VERSION_NAMESPACE
@@ -53,7 +55,7 @@ public:
     /// Trivial constructor, the vector is NOT initialized
     Vec3() {}
 
-    /// Constructor with one argument, e.g.   Vec3f v(0);
+    /// @brief Construct a vector all of whose components have the given value.
     explicit Vec3(T val) { this->mm[0] = this->mm[1] = this->mm[2] = val; }
 
     /// Constructor with three arguments, e.g.   Vec3d v(1,2,3);
@@ -83,6 +85,16 @@ public:
         this->mm[2] = static_cast<T>(v[2]);
     }
 
+    /// @brief Construct a vector all of whose components have the given value,
+    /// which may be of an arithmetic type different from this vector's value type.
+    /// @details Type conversion warnings are suppressed.
+    template<typename Other>
+    explicit Vec3(Other val,
+        typename std::enable_if<std::is_arithmetic<Other>::value, Conversion>::type = Conversion{})
+    {
+        this->mm[0] = this->mm[1] = this->mm[2] = static_cast<T>(val);
+    }
+
     /// @brief Construct a Vec3 from another Vec3 with a possibly different value type.
     /// @details Type conversion warnings are suppressed.
     template<typename Other>
@@ -328,7 +340,7 @@ public:
         return *this;
     }
 
-    /// Return a reference to itsef after the exponent has been
+    /// Return a reference to itself after the exponent has been
     /// applied to all the vector components.
     inline const Vec3<T>& exp()
     {
@@ -338,12 +350,28 @@ public:
         return *this;
     }
 
+    /// Return a reference to itself after log has been
+    /// applied to all the vector components.
+    inline const Vec3<T>& log()
+    {
+        this->mm[0] = std::log(this->mm[0]);
+        this->mm[1] = std::log(this->mm[1]);
+        this->mm[2] = std::log(this->mm[2]);
+        return *this;
+    }
+
     /// Return the sum of all the vector components.
     inline T sum() const
     {
         return this->mm[0] + this->mm[1] + this->mm[2];
     }
 
+    /// Return the product of all the vector components.
+    inline T product() const
+    {
+        return this->mm[0] * this->mm[1] * this->mm[2];
+    }
+
     /// this = normalized this
     bool normalize(T eps = T(1.0e-7))
     {
@@ -373,6 +401,13 @@ public:
         return *this / len;
     }
 
+    /// return normalized this, or (1, 0, 0) if this is null vector
+    Vec3<T> unitSafe() const
+    {
+        T l2 = lengthSqr();
+        return l2 ? *this / static_cast<T>(sqrt(l2)) : Vec3<T>(1, 0 ,0);
+    }
+
     // Number of cols, rows, elements
     static unsigned numRows() { return 1; }
     static unsigned numColumns() { return 3; }
@@ -411,13 +446,13 @@ public:
             l = this->mm[0]*this->mm[0] + this->mm[2]*this->mm[2];
             l = static_cast<T>(T(1)/sqrt(double(l)));
             u.mm[0] = -this->mm[2]*l;
-            u.mm[1] = (T)0.0;
+            u.mm[1] = T(0);
             u.mm[2] = +this->mm[0]*l;
         } else {
             // W.y or W.z is the largest magnitude component, swap them
             l = this->mm[1]*this->mm[1] + this->mm[2]*this->mm[2];
             l = static_cast<T>(T(1)/sqrt(double(l)));
-            u.mm[0] = (T)0.0;
+            u.mm[0] = T(0);
             u.mm[1] = +this->mm[2]*l;
             u.mm[2] = -this->mm[1]*l;
         }
@@ -425,23 +460,25 @@ public:
         return u;
     }
 
-    /// True if a Nan is present in vector
-    bool isNan() const { return isnan(this->mm[0]) || isnan(this->mm[1]) || isnan(this->mm[2]); }
-
-    /// True if an Inf is present in vector
-    bool isInfinite() const
+    /// Return a vector with the components of this in ascending order
+    Vec3<T> sorted() const
     {
-        return isinf(this->mm[0]) || isinf(this->mm[1]) || isinf(this->mm[2]);
+        Vec3<T> r(*this);
+        if( r.mm[0] > r.mm[1] ) std::swap(r.mm[0], r.mm[1]);
+        if( r.mm[1] > r.mm[2] ) std::swap(r.mm[1], r.mm[2]);
+        if( r.mm[0] > r.mm[1] ) std::swap(r.mm[0], r.mm[1]);
+        return r;
     }
 
-    /// True if all no Nan or Inf values present
-    bool isFinite() const
+    /// Return the vector (z, y, x)
+    Vec3<T> reversed() const
     {
-        return finite(this->mm[0]) && finite(this->mm[1]) && finite(this->mm[2]);
+        return Vec3<T>(this->mm[2], this->mm[1], this->mm[0]);
     }
 
     /// Predefined constants, e.g.   Vec3d v = Vec3d::xNegAxis();
     static Vec3<T> zero() { return Vec3<T>(0, 0, 0); }
+    static Vec3<T> ones() { return Vec3<T>(1, 1, 1); }
 };
 
 
@@ -563,21 +600,6 @@ isApproxEqual(const Vec3<T>& a, const Vec3<T>& b, const Vec3<T>& eps)
            isApproxEqual(a.z(), b.z(), eps.z());
 }
 
-template<typename T>
-inline bool
-isFinite(const Vec3<T>& v)
-{
-    return isFinite(v[0]) && isFinite(v[1]) && isFinite(v[2]);
-}
-
-/// Return @c true if all components are exactly equal to zero.
-template<typename T>
-inline bool
-isZero(const Vec3<T>& v)
-{
-    return isZero(v[0]) && isZero(v[1]) && isZero(v[2]);
-}
-
 template<typename T>
 inline Vec3<T>
 Abs(const Vec3<T>& v)
@@ -645,6 +667,11 @@ inline Vec3<T> maxComponent(const Vec3<T> &v1, const Vec3<T> &v2)
 template <typename T>
 inline Vec3<T> Exp(Vec3<T> v) { return v.exp(); }
 
+/// @brief Return a vector with log applied to each of
+/// the components of the input vector.
+template <typename T>
+inline Vec3<T> Log(Vec3<T> v) { return v.log(); }
+
 typedef Vec3<int32_t>   Vec3i;
 typedef Vec3<uint32_t>  Vec3ui;
 typedef Vec3<float>     Vec3s;
@@ -656,6 +683,6 @@ typedef Vec3<double>    Vec3d;
 
 #endif // OPENVDB_MATH_VEC3_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
