@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -32,17 +32,17 @@
 ///
 /// @author Ken Museth
 ///
-/// @brief  Concurrent page-based linear data structure with O(1)
-///         random access and std-compliant iterators. It is
-///         primarily intended for applications that involve
-///         multi-threading of dynamically growing linear arrays with
-///         fast random access. 
+/// @brief  Concurrent, page-based, dynamically-sized linear data
+///         structure with O(1) random access and STL-compliant
+///         iterators. It is primarily intended for applications
+///         that involve multi-threading push_back of (a possibly
+///         unkown number of) elements into a dynamically growing
+///         linear array, and fast random access to said elements.
 
 #ifndef OPENVDB_UTIL_PAGED_ARRAY_HAS_BEEN_INCLUDED
 #define OPENVDB_UTIL_PAGED_ARRAY_HAS_BEEN_INCLUDED
 
-
-#include <deque>
+#include <vector>
 #include <cassert>
 #include <iostream>
 #include <algorithm>// std::swap
@@ -59,13 +59,13 @@ namespace util {
 ////////////////////////////////////////
 
 
-/// @brief   Concurrent page-based linear data structure with O(1)
-///          random access and std-compliant iterators. It is
-///          primarily intended for applications that involve
-///          multi-threading of dynamically growing linear arrays with
-///          fast random access. 
+/// @brief   Concurrent, page-based, dynamically-sized linear data structure
+///          with O(1) random access and STL-compliant iterators. It is
+///          primarily intended for applications that concurrently insert
+///          (a possibly unkown number of) elements into a dynamically
+///          growing linear array, and fast random access to said elements.
 ///
-/// @note    Multiple threads can grow the page-table and push_back  
+/// @note    Multiple threads can grow the page-table and push_back
 ///          new elements concurrently. A ValueBuffer provides accelerated
 ///          and threadsafe push_back at the cost of potentially re-ordering
 ///          elements (when multiple instances are used).
@@ -75,95 +75,89 @@ namespace util {
 ///          capacity is out-grown and new pages are allocated. The
 ///          size of the pages can be controlled with the Log2PageSize
 ///          template parameter (defaults to 1024 elements of type ValueT).
+///          The TableT template parameter is used to define the data
+///          structure for the page table. The default, std::vector,
+///          offers fast random access in exchange for slower
+///          push_back, whereas std:deque offers faster push_back but
+///          slower random access.
 ///
 /// There are three fundamentally different ways to insert elements to
 /// this container - each with different advanteges and disadvanteges.
 ///
-/// The simplest way to insert elements is to use PagedArray::push_back e.g.   
+/// The simplest way to insert elements is to use PagedArray::push_back e.g.
 /// @code
-///   PagedArray<int> array;
-///   for (int i=0; i<100000; ++i) array.push_back(i);
+///   PagedArray<size_t> array;
+///   for (size_t i=0; i<100000; ++i) array.push_back(i);
 /// @endcode
-/// or with tbb task-based multi-threading
+/// or with TBB task-based multi-threading
 /// @code
-/// struct Functor1 {
-///   Functor1(int n, PagedArray<int>& _array) : array(&_array) {
-///     tbb::parallel_for(tbb::blocked_range<int>(0, n, PagedArray<int>::pageSize()), *this);
-///   }
-///   void operator()(const tbb::blocked_range<int>& r) const {
-///      for (int i=r.begin(), n=r.end(); i!=n; ++i) array->push_back(i);
-///   }
-///   PagedArray<int>* array;
-/// };    
-/// PagedArray<int> array;   
-/// Functor1 tmp(10000, array);  
-/// @endcode    
+///   PagedArray<size_t> array;
+///   tbb::parallel_for(
+///       tbb::blocked_range<size_t>(0, 10000, array.pageSize()),
+///       [&array](const tbb::blocked_range<size_t>& range) {
+///           for (size_t i=range.begin(); i!=range.end(); ++i) array.push_back(i);
+///       }
+///   );
+/// @endcode
 /// PagedArray::push_back has the advantage that it's thread-safe and
 /// preserves the ordering of the inserted elements. In fact it returns
 /// the linear offset to the added element which can then be used for
 /// fast O(1) random access. The disadvantage is it's the slowest of
 /// the three different ways of inserting elements.
 ///
-/// The fastest way (by far) to insert elements is to use one (or
-/// more) instances of a PagedArray::ValueBuffer, e.g.
+/// The fastest way (by far) to insert elements by means of a PagedArray::ValueBuffer, e.g.
 /// @code
-///   PagedArray<int> array;    
-///   PagedArray<int>::ValueBuffer buffer(array);
-///   for (int i=0; i<100000; ++i) buffer.push_back(i);    
-///   buffer.flush();    
+///   PagedArray<size_t> array;
+///   PagedArray<size_t>::ValueBuffer buffer(array);
+///   for (size_t i=0; i<100000; ++i) buffer.push_back(i);
+///   buffer.flush();
 /// @endcode
-/// or    
+/// or
 /// @code
-///   PagedArray<int> array;
-///   {//local scope of a single thread   
-///     PagedArray<int>::ValueBuffer buffer(array);
-///     for (int i=0; i<100000; ++i) buffer.push_back(i);    
-///   }    
-/// @endcode
-/// or with tbb task-based multi-threading
-/// @code
-/// struct Functor2 {
-///   Functor2(int n, PagedArray<int>& array) : buffer(array) {
-///     tbb::parallel_for(tbb::blocked_range<int>(0, n, PagedArray<int>::pageSize()), *this);
+///   PagedArray<size_t> array;
+///   {
+///       //local scope of a single thread
+///       PagedArray<size_t>::ValueBuffer buffer(array);
+///       for (size_t i=0; i<100000; ++i) buffer.push_back(i);
 ///   }
-///   void operator()(const tbb::blocked_range<int>& r) const {
-///      for (int i=r.begin(), n=r.end(); i!=n; ++i) buffer.push_back(i);
-///   }
-///   mutable typename PagedArray<int>::ValueBuffer buffer;
-/// };
-/// PagedArray<int> array;    
-/// Functor2 tmp(10000, array);  
 /// @endcode
-/// or with tbb Thread Local Storage for even better performance (due
+/// or with TBB task-based multi-threading
+/// @code
+///   PagedArray<size_t> array;
+///   tbb::parallel_for(
+///       tbb::blocked_range<size_t>(0, 10000, array.pageSize()),
+///       [&array](const tbb::blocked_range<size_t>& range) {
+///           PagedArray<size_t>::ValueBuffer buffer(array);
+///           for (size_t i=range.begin(); i!=range.end(); ++i) buffer.push_back(i);
+///       }
+///   );
+/// @endcode
+/// or with TBB thread-local storage for even better performance (due
 /// to fewer concurrent instantiations of partially full ValueBuffers)
 /// @code
-/// struct Functor3 { 
-///   typedef tbb::enumerable_thread_specific<PagedArray<int>::ValueBuffer> PoolType;     
-///   Functor3(size_t n, PoolType& _pool) : pool(&_pool) {     
-///     tbb::parallel_for(tbb::blocked_range<int>(0, n, PagedArray<int>::pageSize()), *this);
-///   }
-///   void operator()(const tbb::blocked_range<int>& r) const {
-///      PagedArray<int>::ValueBuffer& buffer = pool->local();    
-///      for (int i=r.begin(), n=r.end(); i!=n; ++i) buffer.push_back(i);
-///   }
-///   PoolType* pool;
-/// };   
-/// PagedArray<int> array;   
-/// PagedArray<int>::ValueBuffer exemplar(array);//dummy used for initialization
-/// Functor3::PoolType pool(exemplar);//thread local storage pool of ValueBuffers
-/// Functor3 tmp(10000, pool);
-/// for (Functor3::PoolType::iterator i=pool.begin(); i!=pool.end(); ++i) i->flush();    
+///   PagedArray<size_t> array;
+///   PagedArray<size_t>::ValueBuffer exemplar(array);//dummy used for initialization
+///   tbb::enumerable_thread_specific<PagedArray<size_t>::ValueBuffer>
+///       pool(exemplar);//thread local storage pool of ValueBuffers
+///   tbb::parallel_for(
+///       tbb::blocked_range<size_t>(0, 10000, array.pageSize()),
+///       [&pool](const tbb::blocked_range<size_t>& range) {
+///           PagedArray<size_t>::ValueBuffer &buffer = pool.local();
+///           for (size_t i=range.begin(); i!=range.end(); ++i) buffer.push_back(i);
+///       }
+///   );
+///   for (auto i=pool.begin(); i!=pool.end(); ++i) i->flush();
 /// @endcode
-/// This technique generally outperforms PagedArray::push_back, 
+/// This technique generally outperforms PagedArray::push_back,
 /// std::vector::push_back, std::deque::push_back and even
 /// tbb::concurrent_vector::push_back. Additionally it
-/// is thread-safe as long as each thread has it's own instance of a 
+/// is thread-safe as long as each thread has it's own instance of a
 /// PagedArray::ValueBuffer. The only disadvantage is the ordering of
-/// the elements is undefined if multiple instance of a 
+/// the elements is undefined if multiple instance of a
 /// PagedArray::ValueBuffer are employed. This is typically the case
 /// in the context of multi-threading, where the
 /// ordering of inserts are undefined anyway. Note that a local scope
-/// can be used to guarentee that the ValueBuffer has inerted all its
+/// can be used to guarentee that the ValueBuffer has inserted all its
 /// elements by the time the scope ends. Alternatively the ValueBuffer
 /// can be explicitly flushed by calling ValueBuffer::flush.
 ///
@@ -172,37 +166,50 @@ namespace util {
 /// @code
 ///   PagedArray<int> array;
 ///   array.resize(100000);
-///   for (int i=0; i<100000; ++i) array[i] = i;    
+///   for (int i=0; i<100000; ++i) array[i] = i;
 /// @endcode
 /// or in terms of the random access iterator
 /// @code
 ///   PagedArray<int> array;
 ///   array.resize(100000);
-///   for (PagedArray<int>::Iterator i=array.begin(); i!=array.end(); ++i) *i = i.pos();    
-/// @endcode    
+///   for (auto i=array.begin(); i!=array.end(); ++i) *i = i.pos();
+/// @endcode
 /// While this approach is both fast and thread-safe it suffers from the
 /// major disadvantage that the problem size, i.e. number of elements, needs to
 /// be known in advance. If that's the case you might as well consider
 /// using std::vector or a raw c-style array! In other words the
 /// PagedArray is most useful in the context of applications that
 /// involve multi-threading of dynamically growing linear arrays that
-/// require fast random access. 
-template <typename ValueT, size_t Log2PageSize = 10UL>
-class PagedArray {
+/// require fast random access.
 
-  private:
+template<typename ValueT,
+         size_t Log2PageSize = 10UL,
+         template<typename ...> class TableT = std::vector>
+class PagedArray
+{
+private:
     class Page;
-    typedef std::deque<Page*> PageTableT;
-    
-  public:
+
+#if defined(__INTEL_COMPILER) && __INTEL_COMPILER < 1700
+    // Workaround for ICC 15/16 "too few arguments to template" bug (fixed in ICC 17)
+    using PageTableT = TableT<Page*, std::allocator<Page*>>;
+#else
+    using PageTableT = TableT<Page*>;
+#endif
+
+public:
     typedef ValueT ValueType;
 
     /// @brief Default constructor
-    PagedArray() : mPageTable(), mSize(), mCapacity(0), mGrowthMutex() { mSize = 0; }
+    PagedArray() = default;
 
     /// @brief Destructor removed all allocated pages
     ~PagedArray() { this->clear(); }
-    
+
+    // Disallow copy construction and assignment
+    PagedArray(const PagedArray&) = delete;
+    PagedArray& operator=(const PagedArray&) = delete;
+
     /// @brief Caches values into a local memory Page to improve
     ///        performance of push_back into a PagedArray.
     ///
@@ -212,26 +219,31 @@ class PagedArray {
     /// @warning By design this ValueBuffer is not threadsafe so
     ///          make sure to create an instance per thread!
     class ValueBuffer;
-    
+
     /// Const std-compliant iterator
     class ConstIterator;
 
      /// Non-const std-compliant iterator
     class Iterator;
-  
-    /// @brief  Thread safe insertion, adds a new element at
-    ///         the end and increases the container size by one.
+
+    /// @brief   Thread safe insertion, adds a new element at
+    ///          the end and increases the container size by one and
+    ///          returns the linear offset for the inserted element.
     ///
-    /// @note   Constant time complexity. May allocate a new page.
+    /// @param value value to be added to this PagedArray
+    ///
+    /// @details Constant time complexity. May allocate a new page.
     size_t push_back(const ValueType& value)
     {
         const size_t index = mSize.fetch_and_increment();
-        if (index >= mCapacity) this->grow(index);       
+        if (index >= mCapacity) this->grow(index);
         (*mPageTable[index >> Log2PageSize])[index] = value;
         return index;
     }
 
-    /// @brief Slightly faster then the thread-safe push_back above.
+    /// @brief Slightly faster than the thread-safe push_back above.
+    ///
+    /// @param value value to be added to this PagedArray
     ///
     /// @note For best performance consider using the ValueBuffer!
     ///
@@ -249,6 +261,13 @@ class PagedArray {
 
     /// @brief Returns the last element, decrements the size by one.
     ///
+    /// @deprecated This method will be removed in the near future
+    ///
+    /// @bug This method can produce unexpected results if used
+    /// in combination with a ValueBuffer (due to empty pages).
+    /// However,it can safely be used the push_back methods
+    /// defined above.
+    ///
     /// @details Consider subsequnetly calling shrink_to_fit to
     /// reduce the page table to match the new size.
     ///
@@ -260,20 +279,23 @@ class PagedArray {
     /// expect since the ordering is generally not perserved. Only
     /// PagedArray::push_back preserves the ordering (or a single
     /// instance of a ValueBuffer).
-    ValueType pop_back()
+    OPENVDB_DEPRECATED ValueType pop_back()
     {
         assert(mSize>0);
         --mSize;
         return (*mPageTable[mSize >> Log2PageSize])[mSize];
     }
 
+
     /// @brief Reduce the page table to fix the current size.
     ///
     /// @warning Not thread-safe!
     void shrink_to_fit();
-    
+
     /// @brief Return a reference to the value at the specified offset
     ///
+    /// @param i linear offset of the value to be accessed.
+    ///
     /// @note This random access has constant time complexity.
     ///
     /// @warning It is assumed that the i'th element is already allocated!
@@ -285,6 +307,8 @@ class PagedArray {
 
     /// @brief Return a const-reference to the value at the specified offset
     ///
+    /// @param i linear offset of the value to be accessed.
+    ///
     /// @note This random access has constant time complexity.
     ///
     /// @warning It is assumed that the i'th element is already allocated!
@@ -294,16 +318,57 @@ class PagedArray {
         return (*mPageTable[i>>Log2PageSize])[i];
     }
 
-    /// @brief Set all elements to the specified value
+    /// @brief Set all elements in the page table to the specified value
+    ///
+    /// @param v value to be filled in all the existing pages of this PagedArray.
+    ///
+    /// @note Multi-threaded
     void fill(const ValueType& v)
     {
-        tbb::spin_mutex::scoped_lock lock(mGrowthMutex);
-        Fill tmp(this, v);
+        auto op = [&](const tbb::blocked_range<size_t>& r){
+            for(size_t i=r.begin(); i!=r.end(); ++i) mPageTable[i]->fill(v);
+        };
+        tbb::parallel_for(tbb::blocked_range<size_t>(0, this->pageCount()), op);
     }
 
+    /// @brief Copy the first @a count values in this PageArray into
+    /// a raw c-style array, assuming it to be at least @a count
+    /// elements long.
+    ///
+    /// @param p pointer to an array that will used as the destination of the copy.
+    /// @param count number of elements to be copied.
+    ///
+    bool copy(ValueType *p, size_t count) const
+    {
+        size_t last_page = count >> Log2PageSize;
+        if (last_page >= this->pageCount()) return false;
+        auto op = [&](const tbb::blocked_range<size_t>& r){
+            for (size_t i=r.begin(); i!=r.end(); ++i) {
+                mPageTable[i]->copy(p+i*Page::Size, Page::Size);
+            }
+        };
+        if (size_t m = count & Page::Mask) {//count is not divisible by page size
+            tbb::parallel_for(tbb::blocked_range<size_t>(0, last_page, 32), op);
+            mPageTable[last_page]->copy(p+last_page*Page::Size, m);
+        } else {
+            tbb::parallel_for(tbb::blocked_range<size_t>(0, last_page+1, 32), op);
+        }
+        return true;
+    }
+    void copy(ValueType *p) const { this->copy(p, mSize); }
+
     /// @brief Resize this array to the specified size.
     ///
-    /// @note This will grow or shrink the page table.
+    /// @param size number of elements that this PageArray will contain.
+    ///
+    /// @details Will grow or shrink the page table to contain
+    /// the specified number of elements. It will affect the size(),
+    /// iteration will go over all those elements, push_back will
+    /// insert after them and operator[] can be used directly access
+    /// them.
+    ///
+    /// @note No reserve method is implemented due to efficiency concerns
+    /// (especially for the ValueBuffer) from having to deal with empty pages.
     ///
     /// @warning Not thread-safe!
     void resize(size_t size)
@@ -316,8 +381,19 @@ class PagedArray {
         }
     }
 
-    /// @brief Resize this array to the specified size and
-    ///        set all elements to the specified value.
+    /// @brief Resize this array to the specified size and initialize
+    /// all values to @a v.
+    ///
+    /// @param size number of elements that this PageArray will contain.
+    /// @param v value of all the @a size values.
+    ///
+    /// @details Will grow or shrink the page table to contain
+    /// the specified number of elements. It will affect the size(),
+    /// iteration will go over all those elements, push_back will
+    /// insert after them and operator[] can be used directly access them.
+    ///
+    /// @note No reserve method is implemented due to efficiency concerns
+    /// (especially for the ValueBuffer) from having to deal with empty pages.
     ///
     /// @warning Not thread-safe!
     void resize(size_t size, const ValueType& v)
@@ -325,10 +401,10 @@ class PagedArray {
        this->resize(size);
        this->fill(v);
     }
-    
+
     /// @brief Return the number of elements in this array.
     size_t size() const { return mSize; }
-    
+
     /// @brief Return the maximum number of elements that this array
     /// can contain without allocating more memory pages.
     size_t capacity() const { return mCapacity; }
@@ -354,13 +430,13 @@ class PagedArray {
 
     /// @brief Return true if the container contains no elements.
     bool isEmpty() const { return mSize == 0; }
-    
-    /// @brief Return true if the page table is partially full, i.e. the 
+
+    /// @brief Return true if the page table is partially full, i.e. the
     ///        last non-empty page contains less than pageSize() elements.
     ///
     /// @details When the page table is partially full calling merge()
     ///          or using a ValueBuffer will rearrange the ordering of
-    ///          existing elements. 
+    ///          existing elements.
     bool isPartiallyFull() const { return (mSize & Page::Mask) > 0; }
 
     /// @brief  Removes all elements from the array and delete all pages.
@@ -368,7 +444,6 @@ class PagedArray {
     /// @warning Not thread-safe!
     void clear()
     {
-        tbb::spin_mutex::scoped_lock lock(mGrowthMutex);
         for (size_t i=0, n=mPageTable.size(); i<n; ++i) delete mPageTable[i];
         PageTableT().swap(mPageTable);
         mSize     = 0;
@@ -382,18 +457,24 @@ class PagedArray {
     /// past-the-last element.
     ///
     /// @warning Iterator does not point to a valid element and should not
-    /// be dereferenced! 
+    /// be dereferenced!
     Iterator end() { return Iterator(*this, mSize); }
 
+    //@{
     /// @brief Return a const iterator pointing to the first element
     ConstIterator cbegin() const { return ConstIterator(*this, 0); }
+    ConstIterator begin() const { return ConstIterator(*this, 0); }
+    //@}
 
+    //@{
     /// @brief Return a const iterator pointing to the
     /// past-the-last element.
     ///
-    /// @warning Itrator does not point to a valid element and should not
-    /// be dereferenced! 
+    /// @warning Iterator does not point to a valid element and should not
+    /// be dereferenced!
     ConstIterator cend() const { return ConstIterator(*this, mSize); }
+    ConstIterator end() const { return ConstIterator(*this, mSize); }
+    //@}
 
     /// @brief Parallel sort of all the elements in ascending order.
     void sort() { tbb::parallel_sort(this->begin(), this->end(), std::less<ValueT>() ); }
@@ -401,15 +482,19 @@ class PagedArray {
     /// @brief Parallel sort of all the elements in descending order.
     void invSort() { tbb::parallel_sort(this->begin(), this->end(), std::greater<ValueT>()); }
 
+    //@{
     /// @brief Parallel sort of all the elements based on a custom
     /// functor with the api:
     /// @code bool operator()(const ValueT& a, const ValueT& b) @endcode
     /// which returns true if a comes before b.
     template <typename Functor>
-    void sort() { tbb::parallel_sort(this->begin(), this->end(), Functor() ); }
+    void sort(Functor func) { tbb::parallel_sort(this->begin(), this->end(), func ); }
+    //@}
 
     /// @brief Transfer all the elements (and pages) from the other array to this array.
     ///
+    /// @param other non-const reference to the PagedArray that will be merged into this PagedArray.
+    ///
     /// @note The other PagedArray is empty on return.
     ///
     /// @warning The ordering of elements is undefined if this page table is partially full!
@@ -423,19 +508,13 @@ class PagedArray {
              << "\tPage table: " << this->pageCount() << " pages\n"
              << "\tPage size:  " << this->pageSize() << " elements\n"
              << "\tCapacity:   " << this->capacity() << " elements\n"
-             << "\tFootrpint:  " << this->memUsage() << " bytes\n";
+             << "\tFootprint:  " << this->memUsage() << " bytes\n";
       }
 
 private:
-    // Disallow copy construction and assignment
-    PagedArray(const PagedArray&);//not implemented
-    void operator=(const PagedArray&);//not implemented
 
     friend class ValueBuffer;
 
-    // Private class for concurrent fill
-    struct Fill;
-
     void grow(size_t index)
     {
         tbb::spin_mutex::scoped_lock lock(mGrowthMutex);
@@ -446,9 +525,9 @@ private:
     }
 
     void add_full(Page*& page, size_t size);
-    
-    void add_partially_full(Page*& page, size_t size);     
-    
+
+    void add_partially_full(Page*& page, size_t size);
+
     void add(Page*& page, size_t size) {
         tbb::spin_mutex::scoped_lock lock(mGrowthMutex);
         if (size == Page::Size) {//page is full
@@ -458,15 +537,15 @@ private:
         }
     }
     PageTableT mPageTable;//holds points to allocated pages
-    tbb::atomic<size_t> mSize;// current number of elements in array
-    size_t mCapacity;//capacity of array given the current page count
+    tbb::atomic<size_t> mSize{0};// current number of elements in array
+    size_t mCapacity = 0;//capacity of array given the current page count
     tbb::spin_mutex mGrowthMutex;//Mutex-lock required to grow pages
 }; // Public class PagedArray
 
-////////////////////////////////////////////////////////////////////////////////    
-    
-template <typename ValueT, size_t Log2PageSize>
-void PagedArray<ValueT, Log2PageSize>::shrink_to_fit()
+////////////////////////////////////////////////////////////////////////////////
+
+template <typename ValueT, size_t Log2PageSize, template<typename ...> class TableT>
+void PagedArray<ValueT, Log2PageSize, TableT>::shrink_to_fit()
 {
     if (mPageTable.size() > (mSize >> Log2PageSize) + 1) {
         tbb::spin_mutex::scoped_lock lock(mGrowthMutex);
@@ -479,13 +558,13 @@ void PagedArray<ValueT, Log2PageSize>::shrink_to_fit()
     }
 }
 
-template <typename ValueT, size_t Log2PageSize>
-void PagedArray<ValueT, Log2PageSize>::merge(PagedArray& other)
+template <typename ValueT, size_t Log2PageSize, template<typename ...> class TableT>
+void PagedArray<ValueT, Log2PageSize,TableT >::merge(PagedArray& other)
 {
-    if (!other.isEmpty()) {
+    if (&other != this && !other.isEmpty()) {
         tbb::spin_mutex::scoped_lock lock(mGrowthMutex);
         // extract last partially full page if it exists
-        Page* page = NULL;
+        Page* page = nullptr;
         const size_t size = mSize & Page::Mask; //number of elements in the last page
         if ( size > 0 ) {
             page = mPageTable.back();
@@ -501,25 +580,25 @@ void PagedArray<ValueT, Log2PageSize>::merge(PagedArray& other)
         PageTableT().swap(other.mPageTable);
         // add back last partially full page
         if (page) this->add_partially_full(page, size);
-    } 
-}    
+    }
+}
 
-template <typename ValueT, size_t Log2PageSize>
-void PagedArray<ValueT, Log2PageSize>::add_full(Page*& page, size_t size)
+template <typename ValueT, size_t Log2PageSize, template<typename ...> class TableT>
+void PagedArray<ValueT, Log2PageSize, TableT>::add_full(Page*& page, size_t size)
 {
     assert(size == Page::Size);//page must be full
     if (mSize & Page::Mask) {//page-table is partially full
         Page*& tmp = mPageTable.back();
         std::swap(tmp, page);//swap last table entry with page
     }
-    mPageTable.push_back( page );
+    mPageTable.push_back(page);
     mCapacity += Page::Size;
     mSize     += size;
-    page       = NULL;
+    page       = nullptr;
 }
-    
-template <typename ValueT, size_t Log2PageSize>
-void PagedArray<ValueT, Log2PageSize>::add_partially_full(Page*& page, size_t size)
+
+template <typename ValueT, size_t Log2PageSize, template<typename ...> class TableT>
+void PagedArray<ValueT, Log2PageSize, TableT>::add_partially_full(Page*& page, size_t size)
 {
     assert(size > 0 && size < Page::Size);//page must be partially full
     if (size_t m = mSize & Page::Mask) {//page table is also partially full
@@ -533,28 +612,31 @@ void PagedArray<ValueT, Log2PageSize>::add_partially_full(Page*& page, size_t si
         }
     } else {//page table is full so simply append page
         mPageTable.push_back( page );
-        mCapacity += Page::Size;   
-        page       = NULL;
+        mCapacity += Page::Size;
+        page       = nullptr;
     }
     mSize += size;
 }
-    
+
 ////////////////////////////////////////////////////////////////////////////////
 
-// Public member-class of PagedArray    
-template <typename ValueT, size_t Log2PageSize>
-class PagedArray<ValueT, Log2PageSize>::
+// Public member-class of PagedArray
+template <typename ValueT, size_t Log2PageSize, template<typename ...> class TableT>
+class PagedArray<ValueT, Log2PageSize, TableT>::
 ValueBuffer
 {
 public:
-    typedef PagedArray<ValueT, Log2PageSize> PagedArrayType;
+    typedef PagedArray<ValueT, Log2PageSize, TableT> PagedArrayType;
     /// @brief Constructor from a PageArray
     ValueBuffer(PagedArray& parent) : mParent(&parent), mPage(new Page()), mSize(0) {}
     /// @warning This copy-constructor is shallow in the sense that no
     ///          elements are copied, i.e. size = 0.
     ValueBuffer(const ValueBuffer& other) : mParent(other.mParent), mPage(new Page()), mSize(0) {}
     /// @brief Destructor that transfers an buffered values to the parent PagedArray.
-    ~ValueBuffer() { this->flush(); delete mPage; }
+    ~ValueBuffer() { mParent->add(mPage, mSize); delete mPage; }
+
+    ValueBuffer& operator=(const ValueBuffer&) = delete;// disallow copy assignment
+
     /// @brief Add a value to the buffer and increment the size.
     ///
     /// @details If the internal memory page is full it will
@@ -563,14 +645,14 @@ public:
         (*mPage)[mSize++] = v;
         if (mSize == Page::Size) this->flush();
     }
-    /// @brief Manually transfer the values in this buffer to the parent PagedArray.
+    /// @brief Manually transfers the values in this buffer to the parent PagedArray.
     ///
     /// @note This method is also called by the destructor and
-    /// puach_back so it should only be called when manually want to
+    /// push_back so it should only be called if one manually wants to
     /// sync up the buffer with the array, e.g. during debugging.
     void flush() {
         mParent->add(mPage, mSize);
-        if (mPage == NULL) mPage = new Page();
+        if (mPage == nullptr) mPage = new Page();
         mSize = 0;
     }
     /// @brief Return a reference to the parent PagedArray
@@ -578,25 +660,24 @@ public:
     /// @brief Return the current number of elements cached in this buffer.
     size_t size() const { return mSize; }
 private:
-    ValueBuffer& operator=(const ValueBuffer& other);//not implemented
     PagedArray* mParent;
-    Page*       mPage; 
+    Page*       mPage;
     size_t      mSize;
 };// Public class PagedArray::ValueBuffer
-  
+
 ////////////////////////////////////////////////////////////////////////////////
-  
+
 // Const std-compliant iterator
-// Public member-class of PagedArray     
-template <typename ValueT, size_t Log2PageSize>
-class PagedArray<ValueT, Log2PageSize>::
+// Public member-class of PagedArray
+template <typename ValueT, size_t Log2PageSize, template<typename ...> class TableT>
+class PagedArray<ValueT, Log2PageSize, TableT>::
 ConstIterator : public std::iterator<std::random_access_iterator_tag, ValueT>
 {
 public:
     typedef std::iterator<std::random_access_iterator_tag, ValueT> BaseT;
     typedef typename BaseT::difference_type difference_type;
     // constructors and assignment
-    ConstIterator() : mPos(0), mParent(NULL) {}
+    ConstIterator() : mPos(0), mParent(nullptr) {}
     ConstIterator(const PagedArray& parent, size_t pos=0) : mPos(pos), mParent(&parent) {}
     ConstIterator(const ConstIterator& other) : mPos(other.mPos), mParent(other.mParent) {}
     ConstIterator& operator=(const ConstIterator& other) {
@@ -628,25 +709,26 @@ public:
     bool operator< (const ConstIterator& other) const { return mPos <  other.mPos; }
     bool operator> (const ConstIterator& other) const { return mPos >  other.mPos; }
     // non-std methods
-    bool isValid() const { return mParent != NULL && mPos < mParent->size(); }
+    bool isValid() const { return mParent != nullptr && mPos < mParent->size(); }
     size_t pos()   const { return mPos; }
 private:
     size_t            mPos;
     const PagedArray* mParent;
 };// Public class PagedArray::ConstIterator
-  
-////////////////////////////////////////////////////////////////////////////////  
 
-// Public member-class of PagedArray     
-template <typename ValueT, size_t Log2PageSize>
-class PagedArray<ValueT, Log2PageSize>::
+////////////////////////////////////////////////////////////////////////////////
+
+// Non-const std-compliant iterator
+// Public member-class of PagedArray
+template <typename ValueT, size_t Log2PageSize, template<typename ...> class TableT>
+class PagedArray<ValueT, Log2PageSize, TableT>::
 Iterator : public std::iterator<std::random_access_iterator_tag, ValueT>
 {
 public:
     typedef std::iterator<std::random_access_iterator_tag, ValueT> BaseT;
     typedef typename BaseT::difference_type difference_type;
     // constructors and assignment
-    Iterator() : mPos(0), mParent(NULL) {}
+    Iterator() : mPos(0), mParent(nullptr) {}
     Iterator(PagedArray& parent, size_t pos=0) : mPos(pos), mParent(&parent) {}
     Iterator(const Iterator& other) : mPos(other.mPos), mParent(other.mParent) {}
     Iterator& operator=(const Iterator& other) {
@@ -678,7 +760,7 @@ public:
     bool operator< (const Iterator& other) const { return mPos <  other.mPos; }
     bool operator> (const Iterator& other) const { return mPos >  other.mPos; }
     // non-std methods
-    bool isValid() const { return mParent != NULL && mPos < mParent->size(); }
+    bool isValid() const { return mParent != nullptr && mPos < mParent->size(); }
     size_t pos()   const { return mPos; }
   private:
     size_t      mPos;
@@ -688,48 +770,44 @@ public:
 ////////////////////////////////////////////////////////////////////////////////
 
 // Private member-class of PagedArray implementing a memory page
-template <typename ValueT, size_t Log2PageSize>
-class PagedArray<ValueT, Log2PageSize>::
+template <typename ValueT, size_t Log2PageSize, template<typename ...> class TableT>
+class PagedArray<ValueT, Log2PageSize, TableT>::
 Page
 {
 public:
     static const size_t Size = 1UL << Log2PageSize;
     static const size_t Mask = Size - 1UL;
     static size_t memUsage() { return sizeof(ValueT)*Size; }
-    Page() : mData(new ValueT[Size]) {}
+    // Raw memory allocation without any initialization
+    Page() : mData(reinterpret_cast<ValueT*>(new char[sizeof(ValueT)*Size])) {}
     ~Page() { delete [] mData; }
+    Page(const Page&) = delete;//copy construction is not implemented
+    Page& operator=(const Page&) = delete;//copy assignment is not implemented
     ValueT& operator[](const size_t i) { return mData[i & Mask]; }
     const ValueT& operator[](const size_t i) const { return mData[i & Mask]; }
-    void fill(const ValueT& v) { ValueT* p = mData; for (size_t i=Size; i; --i) *p++ = v; }
+    void fill(const ValueT& v) {
+        ValueT* dst = mData;
+        for (size_t i=Size; i; --i) *dst++ = v;
+    }
     ValueT* data() { return mData; }
+    // Copy the first n elements of this Page to dst (which is assumed to large
+    // enough to hold the n elements).
+    void copy(ValueType *dst, size_t n) const {
+        const ValueT* src = mData;
+        for (size_t i=n; i; --i) *dst++ = *src++;
+    }
 protected:
-    Page(const Page& other);//copy construction is not implemented
-    Page& operator=(const Page& rhs);//copy assignment is not implemented
     ValueT* mData;
 };// Private class PagedArray::Page
 
 ////////////////////////////////////////////////////////////////////////////////
 
-// Private member-class of PagedArray implementing concurrent fill of a Page
-template <typename ValueT, size_t Log2PageSize>
-struct PagedArray<ValueT, Log2PageSize>::
-Fill {
-    Fill(PagedArray* _d, const ValueT& _v) : d(_d), v(_v) {
-        tbb::parallel_for(tbb::blocked_range<size_t>(0, d->pageCount()), *this);
-    }
-    void operator()(const tbb::blocked_range<size_t>& r) const {
-        for (size_t i=r.begin(); i!=r.end(); ++i) d->mPageTable[i]->fill(v);
-    }
-    PagedArray* d;
-    const ValueT& v;
-};// Private class PagedArray::Fill
-
 } // namespace util
 } // namespace OPENVDB_VERSION_NAME
 } // namespace openvdb
 
 #endif // OPENVDB_UTIL_PAGED_ARRAY_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2017 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
